Bartender3 = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0", "AceModuleCore-2.0")

--[[
	Layouts
		0 = simple
		1 = enhanced
		2 = advanced
]]

function Bartender3:OnInitialize()
	self:SetDebugging(true)
	self:RegisterDB("BT3DB")
	
	self.defaults = {
		Bars = {
			['*'] = {
				PosX = 0,
				PosY = 0,
				Rows = 1,
				Padding = 0,
				Scale = 1,
				Alpha = 1,
				Hide = false,
				Enable = true,
				Tooltip = true,
				Zoomed = true,
			},
		},
		Layout = 0,
	}
	self:RegisterDefaults('profile', self.defaults)
	
	_, self.playerclass = UnitClass("player")
	
	self.unlock = false
	self.bars, self.actionbuttons = {}, {}
	for i=1,120 do
		self.actionbuttons[i] = "unused"
	end
	if self.db.profile.Layout < 2 then
		for i=1,(6 + (self.db.profile.Layout*4)) do
			local bar = Bartender3.Class.ActionBar:new(i, self.db.profile.Bars[i], self.db.profile.Bars[i].Buttons or 12, (i == 1) and true or false)
			self.bars[i] = bar
		end
	else
		-- TODO Advanced
	end
	
	self:RemoveDefaultUIElements()
end

function Bartender3:RemoveDefaultUIElements()
	-- Hide the main grafics
	MainMenuBarArtFrame:Hide()
	MainMenuBarArtFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
	MainMenuBarArtFrame:UnregisterEvent("VARIABLES_LOADED")
	MainMenuBarArtFrame:UnregisterEvent("BAG_UPDATE")
	MainMenuBarArtFrame:UnregisterEvent("ACTIONBAR_PAGE_CHANGED")
	-- Unregister XP Bar Events
	MainMenuExpBar:UnregisterEvent("PLAYER_ENTERING_WORLD")
	MainMenuExpBar:UnregisterEvent("PLAYER_XP_UPDATE")
	ExhaustionTick:UnregisterEvent("PLAYER_ENTERING_WORLD")
	ExhaustionTick:UnregisterEvent("PLAYER_XP_UPDATE")
	ExhaustionTick:UnregisterEvent("UPDATE_EXHAUSTION")
	ExhaustionTick:UnregisterEvent("PLAYER_LEVEL_UP")
	ExhaustionTick:UnregisterEvent("PLAYER_UPDATE_RESTING")

	MainMenuBar:Hide()
	
	-- temp solution to see the missing buttons while coding :)
	TempButtons = {}
	for i=1,10 do table.insert(TempButtons, getglobal("ShapeshiftButton"..i)) end
	for i=1,10 do table.insert(TempButtons, getglobal("PetActionButton"..i)) end
	table.insert(TempButtons, KeyRingButton)
	table.insert(TempButtons, CharacterBag3Slot)
	table.insert(TempButtons, CharacterBag2Slot)
	table.insert(TempButtons, CharacterBag1Slot)
	table.insert(TempButtons, CharacterBag0Slot)
	table.insert(TempButtons, MainMenuBarBackpackButton)
	table.insert(TempButtons, CharacterMicroButton)
	table.insert(TempButtons, SpellbookMicroButton)
	table.insert(TempButtons, TalentMicroButton)
	table.insert(TempButtons, QuestLogMicroButton)
	table.insert(TempButtons, SocialsMicroButton)
	table.insert(TempButtons, LFGMicroButton)
	table.insert(TempButtons, MainMenuMicroButton)
	table.insert(TempButtons, HelpMicroButton)
	for i,v in pairs(TempButtons) do v:SetParent(UIParent) end
	for i,v in pairs(TempButtons) do v:Show() end

end

function Bartender3:OnEnable()
	for i,v in pairs(self.bars) do
		v:RefreshLayout()
	end
end

function Bartender3:OnProfileEnable()
	for i,v in pairs(self.bars) do
		v:UpdateProfile(self.db.profile.Bars[i])
	end
end

function Bartender3:GetNextButtonID(bar)
	if self.db.profile.Layout < 2 and bar then
		for i=1,12 do
			local button = ((bar-1)*12)+i
			if ( type(self.actionbuttons[button]) == "string"  and self.actionbuttons[button] == "unused" ) then return button end
		end
	end
end

function Bartender3:Unlock()
	for i,v in pairs(self.bars) do
		if v then v:UnlockFrames() end
	end
	self.unlock = true
end

function Bartender3:Lock()
	for i,v in pairs(self.bars) do
		if v then v:LockFrames() end
	end
	self.unlock = nil
end
